@@ -2,6 +2,8 @@
 
 import fcntl
 import os
+import pwd
+import grp
 import sys
 import signal
 import resource
@@ -17,33 +19,25 @@ class Daemonize(object):
     - pid: path to the pidfile.
     - action: your custom function which will be executed after daemonization.
     - keep_fds: optional list of fds which should not be closed.
+    - privileged_action: action that will be executed before drop privileges if user or
+                         group parameter is provided.
+    - user: drop privileges to this user if provided.
+    - group: drop privileges to this group if provided.
     """
-    def __init__(self, app, pid, action, keep_fds=None):
+    def __init__(self, app, pid, action, keep_fds=None, privileged_action=None, user=None, group=None, verbose=False):
         self.app = app
         self.pid = pid
         self.action = action
-        if keep_fds:
-            self.keep_fds = keep_fds
-        else:
-            self.keep_fds = []
+        self.keep_fds = keep_fds or []
+        self.privileged_action = privileged_action or (lambda: ())
+        self.user = user
+        self.group = group
         # Initialize logging.
         self.logger = logging.getLogger(self.app)
         self.logger.setLevel(logging.DEBUG)
         # Display log messages only on defined handlers.
         self.logger.propagate = False
-        # It will work on OS X and Linux. No FreeBSD support, guys, I don't want to import re here
-        # to parse your peculiar platform string.
-        if sys.platform == "darwin":
-            syslog_address = "/var/run/syslog"
-        else:
-            syslog_address = "/dev/log"
-        syslog = handlers.SysLogHandler(syslog_address)
-        syslog.setLevel(logging.INFO)
-        # Try to mimic to normal syslog messages.
-        formatter = logging.Formatter("%(asctime)s %(name)s: %(message)s",
-                                      "%b %e %H:%M:%S")
-        syslog.setFormatter(formatter)
-        self.logger.addHandler(syslog)
+        self.verbose = verbose
 
     def sigterm(self, signum, frame):
         """ sigterm method
@@ -102,6 +96,25 @@ class Daemonize(object):
         os.dup(0)
         os.dup(0)
 
+        # Initialize syslog.
+        # It will work on OS X and Linux. No FreeBSD support, guys, I don't want to import re here
+        # to parse your peculiar platform string.
+        if sys.platform == "darwin":
+            syslog_address = "/var/run/syslog"
+        else:
+            syslog_address = "/dev/log"
+        syslog = handlers.SysLogHandler(syslog_address)
+        if self.verbose:
+            syslog.setLevel(logging.DEBUG)
+        else:
+            syslog.setLevel(logging.INFO)
+        # Try to mimic to normal syslog messages.
+        formatter = logging.Formatter("%(asctime)s %(name)s: %(message)s",
+                                      "%b %e %H:%M:%S")
+        syslog.setFormatter(formatter)
+
+        self.logger.addHandler(syslog)
+
         # Set umask to default to safe file permissions when running as a root daemon. 027 is an
         # octal number which we are typing as 0o27 for Python3 compatibility.
         os.umask(0o27)
@@ -110,15 +123,52 @@ class Daemonize(object):
         # needs to be deleted results in "directory busy" errors.
         os.chdir("/")
 
-        # Create a lockfile so that only one instance of this daemon is running at any time.
-        lockfile = open(self.pid, "w")
-        # Try to get an exclusive lock on the file. This will fail if another process has the file
-        # locked.
-        fcntl.lockf(lockfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
+        # Execute privileged action
+        priviled_action_result = self.privileged_action()
 
-        # Record the process id to the lockfile. This is standard practice for daemons.
-        lockfile.write("%s" % (os.getpid()))
-        lockfile.flush()
+        # Change gid
+        if self.group:
+            try:
+                gid = grp.getgrnam(self.group).gr_gid
+            except KeyError:
+                self.logger.error("Group {0} not found".format(self.group))
+                sys.exit(1)
+            try:
+                os.setgid(gid)
+            except OSError:
+                self.logger.error("Unable to change gid.")
+                sys.exit(1)
+
+        # Change uid
+        if self.user:
+            try:
+                uid = pwd.getpwnam(self.user).pw_uid
+            except KeyError:
+                self.logger.error("User {0} not found.".format(self.user))
+                sys.exit(1)
+            try:
+                os.setuid(uid)
+            except OSError:
+                self.logger.error("Unable to change uid.")
+                sys.exit(1)
+
+        try:
+            # Create a lockfile so that only one instance of this daemon is running at any time.
+            lockfile = open(self.pid, "w")
+        except IOError:
+            self.logger.error("Unable to create a pidfile.")
+            sys.exit(1)
+        try:
+            # Try to get an exclusive lock on the file. This will fail if another process has the file
+            # locked.
+            fcntl.lockf(lockfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            # Record the process id to the lockfile. This is standard practice for daemons.
+            lockfile.write("%s" % (os.getpid()))
+            lockfile.flush()
+        except IOError:
+            self.logger.error("Unable to lock on the pidfile.")
+            os.remove(self.pid)
+            sys.exit(1)
 
         # Set custom action on SIGTERM.
         signal.signal(signal.SIGTERM, self.sigterm)
@@ -126,4 +176,4 @@ class Daemonize(object):
 
         self.logger.warn("Starting daemon.")
 
-        self.action()
+        self.action(*priviled_action_result)
